import click

import robotmk.modes as modes
from robotmk.modes.agent import cli as agent_cli
from robotmk.modes.specialagent import cli as specialagent_cli
from robotmk.modes.robot import cli as robot_cli
from robotmk.modes.output import cli as output_cli


# TODO: options do not work here
# @click.option('--version', is_flag=True, help="Print version and exit")
# @click.option('--verbose', is_flag=True, help="Enable verbose mode")
@click.group(invoke_without_command=True)
@click.pass_context
def main(context):
    print(__name__ + ": " + "(cli main)")
    if context.invoked_subcommand is None:
        modes.run_output()
    else:
        # if context.opts["verbose"]:
        #     print(__name__ + ": " + f"robotmk_agent version: {__version__}")
        # print(__name__ + ": " + f"Invoked subcommand: {context.invoked_subcommand}")
        pass


# --------------------------------------------------
# AGENT


@main.group(
    name="agent",
    help="Execute Robotmk agent (Windows/Linux).",
    invoke_without_command=False,
)
# do not execute without subcommand
def cli_agent():
    print(__name__ + ": " + "(cli_agent)")
    pass


cli_agent.add_command(agent_cli.fg)
cli_agent.add_command(agent_cli.bg)
# cli_agent.add_command(agent_cli.stop)
# cli_agent.add_command(agent_cli.restart)

# --------------------------------------------------
# SPECIAL AGENT


@main.group(
    name="specialagent",
    help="Execute Robotmk as Special Agent (Checkmk).",
    invoke_without_command=True,
)
def cli_specialagent():
    modes.run_specialagent()


cli_specialagent.add_command(specialagent_cli.yyyy)


# --------------------------------------------------
# OUTPUT


@main.group(
    name="output",
    help="Produce Robotmk Agent output for Checkmk.",
    invoke_without_command=True,
)
def cli_output():
    modes.run_output()


cli_output.add_command(output_cli.yyyy)


# --------------------------------------------------
# ROBOT


@main.group(
    name="robot",
    help="Execute a single Robot (=Robot Framework suite).",
    invoke_without_command=True,
)
def cli_robot():
    modes.run_robot()


cli_specialagent.add_command(robot_cli.yyyy)

# --------------------------------------------------
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# SPDX-FileCopyrightText: © 2022 ELABIT GmbH <mail@elabit.de>
# SPDX-License-Identifier: GPL-3.0-or-later


import sys, os, time, atexit, signal
import platform
from abc import ABC, abstractmethod
from pathlib import Path
import re
import click


class RMKAgent:
    def __init__(self, name="robotmk_agent_daemon", pidfile=None, controlled=False):
        self.name = name
        if not pidfile:
            # TODO: find a path that is accessible from insice RCC and outside
            pidfile = "%s.pid" % self.name

            print(__name__ + ": (Daemon init) " + "tmpdir is: %s" % self.tmpdir)
            self.pidfile = self.tmpdir / pidfile
            print(__name__ + ": (Daemon init) " + "Pidfile is: %s" % self.pidfile)
        else:
            self.pidfile = Path(pidfile)
        self.controlled = controlled
        self.lastexecfile_path = self.tmpdir / "robotmk_controller_last_execution"

        # if platform.system() == "Linux":
        #     self.fork_strategy = LinuxStrategy(self)
        # elif platform.system() == "Windows":
        #     self.fork_strategy = WindowsStrategy(self)

    # def daemonize(self):
    #     self.fork_strategy.daemonize()
    #     self.write_and_register_pidfile()

    @property
    def tmpdir(self):
        # if env var does not exist, throw exception
        if not os.getenv("CMK_AGENT_DIR"):
            raise Exception(
                "TBD: Environment variable CMK_AGENT_DIR not set. Please set it to the path of the agent directory."
            )
        return Path(os.environ.get("CMK_AGENT_DIR")) / "tmp"

    @property
    def pid(self):
        return str(os.getpid())

    def get_pid_from_file(self):
        try:
            with open(self.pidfile, "r") as pf:
                pid = int(pf.read().strip())
        except IOError:
            pid = None
        return pid

    def delpid(self):
        os.remove(self.pidfile)

    def running_allowed(self):
        if self.controlled:
            return self.ctrl_file_is_fresh()
        else:
            return True

    def ctrl_file_is_fresh(self):
        mtime = os.path.getmtime(self.lastexecfile_path)
        now = time.time()
        if now - mtime < 10:
            return True
        else:
            return False

    def write_and_register_pidfile(self):
        with open(self.pidfile, "w+") as f:
            f.write(self.pid + "\n")
        # Attention: pidfile gets not deleted if daemon was started with Debugger!
        atexit.register(self.delpid)

    def start(self):
        # Check for a pidfile to see if the daemon already runs
        pid = self.get_pid_from_file()

        if pid:
            msg = "One instance of %s is already running (PID: %d).\n" % (
                self.name,
                int(pid),
            )
            print(__name__ + ": " + msg.format(self.pidfile))
            sys.exit(1)
        else:
            # daemonize according to the strategy
            # self.daemonize()
            # Do the work
            self.run()

    def run(self):
        print(__name__ + ": (start) " + "Starting %s" % self.name)
        while self.running_allowed():
            # DUMMY DAEMON CODE
            print(__name__ + ": " + "Daemon is running ... ")
            for i in range(20):
                if i == 19:
                    # remove all files
                    for file in self.tmpdir.glob("robotmk_output_*.txt"):
                        file.unlink()
                else:
                    filename = "robotmk_output_%d.txt" % i
                    with open(self.tmpdir / filename, "w") as f:
                        f.write("foobar output")
                    time.sleep(0.2)
            # if self.exit_without_controller():
            #     break
        print(
            __name__
            + ": "
            + f"I am not supposed to run anymore (reason: controller file {self.lastexecfile_path} is OUTDATED). Exiting, Bye."
        )

    def stop(self):
        # Check for a pidfile to see if the daemon already runs
        pid = self.get_pid_from_file()

        if not pid:
            message = "pidfile {0} does not exist. " + "Daemon does not seem to run.\n"
            print(__name__ + ": " + message.format(self.pidfile))
            return  # not an error in a restart

        # Try killing the daemon process
        try:
            while 1:
                os.kill(pid, signal.SIGTERM)
                time.sleep(0.1)
        except OSError as err:
            e = str(err.args)
            os.remove(self.pidfile)
            sys.exit()
            # delete
            # e = "(22, 'Falscher Parameter', None, 87, None)"
            # kommt manchmal - abfangen: (13, 'Zugriff verweigert', None, 5, None)
            if e.find("No such process") > 0 or re.match(".*22.*87", e):
                if os.path.exists(self.pidfile):
                    os.remove(self.pidfile)
            else:
                print(__name__ + ": " + str(err.args))
                sys.exit(1)

    def restart(self):
        """Restart the daemon."""
        print(__name__ + ": " + "Restarting daemon ... ")
        self.stop()
        self.start()


# class ForkStrategy(ABC):
#     def __init__(self, daemon):
#         self.daemon = daemon

#     @abstractmethod
#     def daemonize(self):
#         pass


# class LinuxStrategy(ForkStrategy):
#     def daemonize(self):

#         try:
#             # FORK I) the child process
#             pid = os.fork()
#             if pid > 0:
#                 # exit the parent process
#                 sys.exit(0)
#         except OSError as err:
#             sys.stderr.write("fork #1 failed: {0}\n".format(err))
#             sys.exit(1)

#         # executed as child process
#         # decouple from parent environment, start new session
#         # with no controlling terminals
#         os.chdir("/")
#         os.setsid()
#         os.umask(0)

#         # FORK II) the grandchild process
#         try:
#             pid = os.fork()
#             if pid > 0:
#                 # exit the child process
#                 sys.exit(0)
#         except OSError as err:
#             sys.stderr.write("fork #2 failed: {0}\n".format(err))
#             sys.exit(1)

#         # here we are the grandchild process,
#         # daemonize it, connect fds to /dev/null stream
#         sys.stdout.flush()
#         sys.stderr.flush()
#         si = open(os.devnull, "r")
#         so = open(os.devnull, "a+")
#         se = open(os.devnull, "a+")

#         os.dup2(si.fileno(), sys.stdin.fileno())
#         os.dup2(so.fileno(), sys.stdout.fileno())
#         os.dup2(se.fileno(), sys.stderr.fileno())


# class WindowsStrategy(ForkStrategy):
#     def daemonize(self):
#         # On Windows, use ProcessCreationFlags to detach this process from the caller
#         print(__name__ + ": " + "On windows, there is nothing to daemonize....")
#         pass



"""Robot Framework test execution and result parsing for Check_MK"""
__version__ = "0.0.27"

# from robotmk import cli
# import sys

# TODO: some bug in imports... when executed from cli, there should not be a message.

# # check if module was imported with cmdline args
# if __name__ == "robotmk" and len(sys.argv) > 1:
if __name__ == "robotmk":
    print(
        __name__
        + ": "
        + "You have imported robotmk module with sys args => execute cli!"
    )
    import robotmk.cli as cli

    cli.main()
# else:
#     print(__name__ + ": " + "You have just imported the robotmk module! No execution")



from time import sleep


def run_robot():
    print(__name__ + ": " + "Robotmk is starting the robot routine!")


def run_specialagent():
    print(__name__ + ": " + "Robotmk is starting the special agent routine!")


def run_robot():
    print(__name__ + ": " + "Robotmk is starting the robot routine!")


def run_output():
    print(__name__ + ": " + "Robotmk is starting the output routine!")
    # sleep(3)
    # print checkmk agent dummy output
    print(__name__ + ": " + "<<<robotmk:sep(0)>>>")
    print(__name__ + ": " + "foobar output ")
    # touch file in temp directory
    # with open(
    #     "C:\\Users\\vagrant\\AppData\\Local\\Temp\\robotmk_output2.txt", "w"
    # ) as f:
    #     f.write("foobar output")
